@@ -27,9 +27,8 @@
 #include <string.h>
 #include <sys/stat.h>
 
-#include <glib/gi18n-lib.h>
-
 #include <libedataserver/e-data-server-util.h>
+#include <camel/camel-i18n.h>
 
 #include "camel-exchange-folder.h"
 #include "camel-exchange-search.h"
@@ -440,6 +439,7 @@ get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 	CamelStreamFilter *filtered_stream;
 	CamelMimeFilter *crlffilter;
 	GByteArray *ba;
+	const char *from, *from_changed, *sender, **sender_name;
 
 	ba = get_message_data (folder, uid, ex);
 	if (!ba)
@@ -457,8 +457,40 @@ get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 	camel_data_wrapper_construct_from_stream (CAMEL_DATA_WRAPPER (msg),
 						  CAMEL_STREAM (filtered_stream));
 	camel_object_unref (CAMEL_OBJECT (filtered_stream));
-
 	camel_mime_message_set_source (msg, exch->source);
+
+	/*Get the from address*/
+	from = camel_medium_get_header(CAMEL_MEDIUM(msg), "From");
+	const gchar* from_initial = g_strdup(from);
+	
+	/*Check if the sender header exists. The sender header suggests that the mail message is a 
+	delegated message. So Evolution recognizes it as a delegated mail sent by the 
+	"sender" on behalf of "from" */
+
+	if(sender = (const char *) camel_medium_get_header (CAMEL_MEDIUM (msg), "Sender")) {
+
+		struct _camel_header_address *sender_addr = camel_header_address_decode (sender, NULL);
+
+		/* If the sender header has no display name then split up the address field 
+		and display it as the name. Evolution accounts should neccesarily have names specified. 
+		This piece of code is for some other clients which may have the name field as NULL.*/
+		if(sender_addr->name == NULL) {
+			sender_name = g_strsplit (sender_addr->v.addr, "@", 2);        
+			from_changed = g_strdup_printf (_("%s at %s on behalf of %s"), sender_name[0], 
+							sender_name[1], from_initial);
+		}
+
+		else {
+			from_changed=g_strdup_printf (_("%s on behalf of %s"), sender_addr->name, from_initial);
+		}
+
+		camel_medium_set_header (CAMEL_MEDIUM (msg), "From", from_changed);
+	
+		/*Set the Reply-to field since the from field is modified.*/
+		if(!camel_medium_get_header (CAMEL_MEDIUM(msg), "Reply-to")) { 
+			camel_medium_add_header (CAMEL_MEDIUM(msg), "Reply-to", from_initial);	
+		}
+	}
 	fix_broken_multipart_related (CAMEL_MIME_PART (msg));
 	return msg;
 }
@@ -750,7 +782,8 @@ find_parent (CamelExchangeFolder *exch, const char *thread_index)
 void
 camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 				   const char *uid, guint32 flags,
-				   guint32 size, const char *headers)
+				   guint32 size, const char *headers,
+				   const char *href)
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
 	CamelMessageInfo *info;
@@ -793,6 +826,7 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 	info->uid = g_strdup (uid);
 	einfo->info.flags = flags;
 	einfo->info.size = size;
+	einfo->href = g_strdup (href);
 
 	camel_folder_summary_add (folder->summary, info);
 
@@ -999,19 +1033,19 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 	CamelExchangeFolder *exch = (CamelExchangeFolder *)folder;
 	const char *short_name;
 	char *summary_file, *journal_file, *path;
-	GPtrArray *summary, *uids;
+	GPtrArray *summary, *uids, *hrefs;
 	GByteArray *flags;
 	guint32 folder_flags;
 	CamelMessageInfo *info;
 	CamelExchangeMessageInfo *einfo;
-	int i, len;
+	int i, len = 0;
 
 	short_name = strrchr (name, '/');
 	if (!short_name++)
 		short_name = name;
 	camel_folder_construct (folder, parent, name, short_name);
 
-	if (e_util_mkdir_hier (folder_dir, S_IRWXU) != 0) {
+	if (g_mkdir_with_parents (folder_dir, S_IRWXU) != 0) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Could not create directory %s: %s"),
 				      folder_dir, g_strerror (errno));
@@ -1078,11 +1112,14 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		g_ptr_array_set_size (uids, summary->len);
 		flags = g_byte_array_new ();
 		g_byte_array_set_size (flags, summary->len);
+		hrefs = g_ptr_array_new ();
+		g_ptr_array_set_size (hrefs, summary->len);
 
 		for (i = 0; i < summary->len; i++) {
 			info = summary->pdata[i];
 			uids->pdata[i] = (char *)camel_message_info_uid (info);
 			flags->data[i] = ((CamelMessageInfoBase *)info)->flags & CAMEL_EXCHANGE_SERVER_FLAGS;
+			hrefs->pdata[i] = ((CamelExchangeMessageInfo *)info)->href;
 			//camel_tag_list_free (&((CamelMessageInfoBase *)info)->user_tags);
 		}
 
@@ -1092,6 +1129,8 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 				      CAMEL_STUB_ARG_UINT32, create,
 				      CAMEL_STUB_ARG_STRINGARRAY, uids,
 				      CAMEL_STUB_ARG_BYTEARRAY, flags,
+				      CAMEL_STUB_ARG_STRINGARRAY, hrefs,
+				      CAMEL_STUB_ARG_UINT32, CAMEL_EXCHANGE_SUMMARY(folder->summary)->high_article_num,
 				      CAMEL_STUB_ARG_RETURN,
 				      CAMEL_STUB_ARG_UINT32, &folder_flags,
 				      CAMEL_STUB_ARG_STRING, &exch->source,
@@ -1099,10 +1138,15 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		camel_operation_end (NULL);
 		g_ptr_array_free (uids, TRUE);
 		g_byte_array_free (flags, TRUE);
+		g_ptr_array_free (hrefs, TRUE);
 		camel_folder_free_summary (folder, summary);
 		if (!ok)
 			return FALSE;
 
+		if (len)
+			return TRUE;
+
+
 		if (folder_flags & CAMEL_STUB_FOLDER_FILTER)
 			folder->folder_flags |= CAMEL_FOLDER_FILTER_RECENT;
 		if (folder_flags & CAMEL_STUB_FOLDER_FILTER_JUNK)
