@@ -28,15 +28,17 @@
 #include <string.h>
 #include <stdlib.h>
 
-#include <camel/camel-exchange-summary.h>
 #include <camel/camel-file-utils.h>
 #include <camel/camel-offline-store.h>
 
 #include "camel-stub.h"
 #include "camel-exchange-folder.h"
 #include "camel-exchange-journal.h"
+#include "camel-exchange-summary.h"
 
-#define CAMEL_EXCHANGE_SUMMARY_VERSION (1)
+#define CAMEL_EXCHANGE_SUMMARY_VERSION (2)
+
+#define d(x) (x)
 
 static int header_load (CamelFolderSummary *summary, FILE *in);
 static int header_save (CamelFolderSummary *summary, FILE *out);
@@ -48,6 +50,8 @@ static int               message_info_save (CamelFolderSummary *summary,
 					    CamelMessageInfo *info);
 static CamelMessageInfo *message_info_new_from_header  (CamelFolderSummary *summary,
 							struct _camel_header_raw *h);
+static gboolean check_for_trash (CamelFolder *folder);
+static gboolean expunge_mail (CamelFolder *folder, CamelMessageInfo *info);
 
 static gboolean info_set_flags(CamelMessageInfo *info, guint32 flags, guint32 set);
 static gboolean info_set_user_tag(CamelMessageInfo *info, const char *name, const char *value);
@@ -130,20 +134,35 @@ static int
 header_load (CamelFolderSummary *summary, FILE *in)
 {
 	CamelExchangeSummary *exchange = (CamelExchangeSummary *) summary;
-	guint32 version, readonly;
+	guint32 version, readonly, high_article_num = 0;
 	
 	if (CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->summary_header_load (summary, in) == -1)
 		return -1;
 	
 	if (camel_file_util_decode_uint32 (in, &version) == -1)
 		return -1;
-	if (version > CAMEL_EXCHANGE_SUMMARY_VERSION)
-		return -1;
 
 	if (camel_file_util_decode_uint32 (in, &readonly) == -1)
 		return -1;
+
+	/* Old summary file - We need to migrate.  Migration automagically happens when 
+	   camel_folder_summary_save is called 
+	*/
+	if (camel_file_util_decode_uint32 (in, &high_article_num) == -1) {
+		if (version > CAMEL_EXCHANGE_SUMMARY_VERSION)
+			return -1;
+	}
+
+	/* During migration we will not have high_article_num stored in the summary and
+	   essentially we will end up computing it atleast once.
+	*/
 	exchange->readonly = readonly;
+	exchange->high_article_num = high_article_num;
+	exchange->version = version;
 	
+
+	d(g_print ("%s:%s:%d: high_article_num = [%d]\n", __FILE__, __PRETTY_FUNCTION__, __LINE__, high_article_num));
+
 	return 0;
 }
 
@@ -155,12 +174,17 @@ header_save (CamelFolderSummary *summary, FILE *out)
 	if (CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->summary_header_save (summary, out) == -1)
 		return -1;
 	
-	if (camel_file_util_encode_uint32 (out, CAMEL_EXCHANGE_SUMMARY_VERSION) == -1)
+	if (camel_file_util_encode_uint32 (out, exchange->version) == -1)
 		return -1;
 
 	if (camel_file_util_encode_uint32 (out, exchange->readonly) == -1)
 		return -1;
 	
+	if (camel_file_util_encode_uint32 (out, exchange->high_article_num) == -1)
+		return -1;
+
+	d(g_print ("%s:%s:%d: high_article_num = [%d]\n", __FILE__, __PRETTY_FUNCTION__, __LINE__, exchange->high_article_num));
+
 	return 0;
 }
 
@@ -169,7 +193,7 @@ message_info_load (CamelFolderSummary *summary, FILE *in)
 {
 	CamelMessageInfo *info;
 	CamelExchangeMessageInfo *einfo;
-	char *thread_index;
+	char *thread_index, *href = NULL;
 
 	info = CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->message_info_load (summary, in);
 	if (info) {
@@ -182,6 +206,17 @@ message_info_load (CamelFolderSummary *summary, FILE *in)
 			einfo->thread_index = thread_index;
 		else
 			g_free (thread_index);
+
+		/* Old summary file - We need to migrate.  Migration automagically happens when 
+		   camel_folder_summary_save is called 
+		*/
+		if (camel_file_util_decode_string (in, &href) == -1) {
+			if (CAMEL_EXCHANGE_SUMMARY (summary)->version > CAMEL_EXCHANGE_SUMMARY_VERSION)
+				goto error;
+		}
+		
+		einfo->href = href;
+		d(g_print ("%s:%s:%d: einfo->href = [%s]\n", __FILE__, __PRETTY_FUNCTION__, __LINE__, einfo->href));
 	}
 
 	return info;
@@ -198,7 +233,15 @@ message_info_save (CamelFolderSummary *summary, FILE *out, CamelMessageInfo *inf
 	if (CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->message_info_save (summary, out, info) == -1)
 		return -1;
 
-	return camel_file_util_encode_string (out, einfo->thread_index ? einfo->thread_index : "");
+	if (camel_file_util_encode_string (out, einfo->thread_index ? einfo->thread_index : "") == -1)
+		return -1;
+	
+	if (camel_file_util_encode_string (out, einfo->href ? einfo->href : "") == -1)
+		return -1;
+
+	d(g_print ("%s:%s:%d: einfo->href = [%s]\n", __FILE__, __PRETTY_FUNCTION__, __LINE__, einfo->href));
+
+	return 0;
 }
 
 static CamelMessageInfo *
@@ -220,10 +263,46 @@ message_info_new_from_header (CamelFolderSummary *summary, struct _camel_header_
 	return info;
 }
 
+static gboolean
+check_for_trash (CamelFolder *folder)
+{
+	CamelStore *store = (CamelStore *) folder->parent_store;
+	CamelException lex;
+	CamelFolder *trash;
+	
+	camel_exception_init (&lex);
+	trash = camel_store_get_trash (store, &lex);
+
+	if (camel_exception_is_set (&lex) || !trash)
+		return FALSE;
+
+	return folder == trash;
+}
+
+static gboolean
+expunge_mail (CamelFolder *folder, CamelMessageInfo *info)
+{
+	CamelExchangeFolder *exchange_folder = (CamelExchangeFolder *) folder;
+	GPtrArray *uids = g_ptr_array_new ();
+	char *uid = g_strdup (info->uid);
+	CamelException lex;
+
+	g_ptr_array_add (uids, uid);
+	
+	camel_exception_init (&lex);
+	camel_stub_send (exchange_folder->stub, &lex,
+			 CAMEL_STUB_CMD_EXPUNGE_UIDS,
+			 CAMEL_STUB_ARG_FOLDER, folder->full_name,
+			 CAMEL_STUB_ARG_STRINGARRAY, uids,
+			 CAMEL_STUB_ARG_END);
+	
+	g_ptr_array_free (uids, TRUE);
+	return camel_exception_is_set (&lex);
+}
+
 static gboolean
 info_set_flags(CamelMessageInfo *info, guint32 flags, guint32 set)
 {
-	int res;
 	CamelFolder *folder = (CamelFolder *) info->summary->folder;
 	CamelOfflineStore *store = (CamelOfflineStore *) folder->parent_store;
 
@@ -231,28 +310,36 @@ info_set_flags(CamelMessageInfo *info, guint32 flags, guint32 set)
 		return FALSE;
 
 	if (store->state != CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
-		res = CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_set_flags(info, flags, set);
-		if (res && info->summary->folder && info->uid) {
-			camel_stub_send_oneway (((CamelExchangeFolder *)info->summary->folder)->stub,
-						CAMEL_STUB_CMD_SET_MESSAGE_FLAGS,
-						CAMEL_STUB_ARG_FOLDER, info->summary->folder->full_name,
-						CAMEL_STUB_ARG_STRING, info->uid,
-						CAMEL_STUB_ARG_UINT32, set,
-						CAMEL_STUB_ARG_UINT32, flags,
-						CAMEL_STUB_ARG_END);
+		if (folder && info->uid) {
+			if ((flags & set & CAMEL_MESSAGE_DELETED) &&
+			    check_for_trash (folder)) {
+				return expunge_mail (folder, info);
+			} else {
+				camel_stub_send_oneway (((CamelExchangeFolder *)folder)->stub,
+							CAMEL_STUB_CMD_SET_MESSAGE_FLAGS,
+							CAMEL_STUB_ARG_FOLDER, folder->full_name,
+							CAMEL_STUB_ARG_STRING, info->uid,
+							CAMEL_STUB_ARG_UINT32, set,
+							CAMEL_STUB_ARG_UINT32, flags,
+							CAMEL_STUB_ARG_END);
+				return CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_set_flags(info, flags, set);
+			}
 		}
 	}
 	else {
-		res = CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_set_flags(info, flags, set);
-		if(res && info->summary->folder && info->uid) {
-			CamelExchangeFolder *exchange_folder = (CamelExchangeFolder *) info->summary->folder;
-			CamelExchangeJournal *journal = (CamelExchangeJournal *) exchange_folder->journal;
-			camel_exchange_journal_delete (journal, info->uid, flags, set, NULL);
+		if(folder && info->uid) {
+			if ((flags & set & CAMEL_MESSAGE_DELETED) &&
+			    check_for_trash (folder)) {
+				/* FIXME: should add a separate journal entry for this case. */ ;
+			} else {
+				CamelExchangeFolder *exchange_folder = (CamelExchangeFolder *) folder;
+				CamelExchangeJournal *journal = (CamelExchangeJournal *) exchange_folder->journal;
+				camel_exchange_journal_delete (journal, info->uid, flags, set, NULL);
+				return CAMEL_FOLDER_SUMMARY_CLASS (parent_class)->info_set_flags(info, flags, set);
+			}
 		}
-		else
-			return FALSE;
 	}
-	return res;
+	return FALSE;
 }
 
 static gboolean
@@ -316,6 +403,43 @@ camel_exchange_summary_set_readonly (CamelFolderSummary *summary,
 	es->readonly = readonly;
 }
 
+/**
+ * camel_exchange_summary_get_article_num:
+ * @summary: the summary
+ *
+ * Returns the highest article number of a message present in the folder represented by @summary.
+ *
+ * Return value: Highest article number for a message present in the folder.
+ **/
+guint32
+camel_exchange_summary_get_article_num (CamelFolderSummary *summary)
+{
+	g_return_val_if_fail (CAMEL_IS_EXCHANGE_SUMMARY (summary), FALSE);
+
+	return CAMEL_EXCHANGE_SUMMARY (summary)->high_article_num;
+}
+
+/**
+ * camel_exchange_summary_set_article_num:
+ * @summary: the summary
+ * @article_num: Highest article number of a message present in the folder.
+ *
+ * Sets @summary's high-article-number to @article_num. 
+ **/
+void
+camel_exchange_summary_set_article_num (CamelFolderSummary *summary,
+					guint32 article_num)
+{
+	CamelExchangeSummary *es;
+
+	g_return_if_fail (CAMEL_IS_EXCHANGE_SUMMARY (summary));
+
+	es = CAMEL_EXCHANGE_SUMMARY (summary);
+	if (!es->readonly)
+		camel_folder_summary_touch (summary);
+	es->high_article_num = article_num;
+}
+
 /**
  * camel_exchange_summary_add_offline:
  * @summary: the summary
