@@ -94,9 +94,15 @@ connection_handler (GIOChannel *source, GIOCondition condition, gpointer data)
 	switch (command) {
 	case CAMEL_STUB_CMD_CONNECT:
 	{
+		char *pwd
 		d(printf("CONNECT\n"));
 		g_object_ref (stub);
-		MS_CLASS (stub)->connect (stub);
+		if (!mail_stub_read_args (stub,
+					  CAMEL_STUB_ARG_STRING, &pwd,
+					  CAMEL_STUB_ARG_END))
+			goto comm_fail;
+		MS_CLASS (stub)->connect (stub, pwd);
+		g_free (pwd);
 		break;
 	}
 
@@ -105,6 +111,8 @@ connection_handler (GIOChannel *source, GIOCondition condition, gpointer data)
 		char *folder_name;
 		GPtrArray *uids;
 		GByteArray *flags;
+		GPtrArray *hrefs;
+		guint32 high_article_num;
 		guint32 create;
 
 		if (!mail_stub_read_args (stub,
@@ -112,12 +120,14 @@ connection_handler (GIOChannel *source, GIOCondition condition, gpointer data)
 					  CAMEL_STUB_ARG_UINT32, &create,
 					  CAMEL_STUB_ARG_STRINGARRAY, &uids,
 					  CAMEL_STUB_ARG_BYTEARRAY, &flags,
+					  CAMEL_STUB_ARG_STRINGARRAY, &hrefs,
+					  CAMEL_STUB_ARG_UINT32, &high_article_num,
 					  CAMEL_STUB_ARG_END))
 			goto comm_fail;
 		d(printf("GET_FOLDER %s\n", folder_name));
 		g_object_ref (stub);
 		MS_CLASS (stub)->get_folder (stub, folder_name, create,
-					     uids, flags);
+					     uids, flags, hrefs, high_article_num);
 		g_free (folder_name);
 		free_string_array (uids);
 		g_byte_array_free (flags, TRUE);
@@ -196,7 +206,7 @@ connection_handler (GIOChannel *source, GIOCondition condition, gpointer data)
 		d(printf("APPEND_MESSAGE %s %lu %s\n", folder_name,
 			 (gulong)flags, subject));
 		g_object_ref (stub);
-		MS_CLASS (stub)->append_message (stub, folder_name, flags, subject, body->data, body->len);
+		MS_CLASS (stub)->append_message (stub, folder_name, flags, subject, (char *) body->data, body->len);
 		g_free (folder_name);
 		g_free (subject);
 		g_byte_array_free (body, TRUE);
@@ -339,7 +349,7 @@ connection_handler (GIOChannel *source, GIOCondition condition, gpointer data)
 			 from, recips->len));
 		g_object_ref (stub);
 		MS_CLASS (stub)->send_message (stub, from, recips,
-					       body->data, body->len);
+					       (char *) body->data, body->len);
 		g_free (from);
 		free_string_array (recips);
 		g_byte_array_free (body, TRUE);
@@ -541,7 +551,7 @@ mail_stub_read_args (MailStub *stub, ...)
 		case CAMEL_STUB_ARG_UINT32ARRAY:
 		{
 			GArray **arr = va_arg (ap, GArray **);
-			int i, len, unread_count;
+			guint32 i, len, unread_count;
 			status = camel_stub_marshal_decode_uint32 (stub->cmd, &len);
 			if (status == -1)
 				break;
@@ -635,7 +645,7 @@ mail_stub_return_data (MailStub *stub, CamelStubRetval retval, ...)
 			int len = va_arg (ap, int);
 			GByteArray ba;
 
-			ba.data = data;
+			ba.data = (guint8 *) data;
 			ba.len = len;
 			camel_stub_marshal_encode_bytes (marshal, &ba);
 			break;
